/*                                                                      
    This file is part of KAddressBook.                                  
    Copyright (c) 2002 Mike Pilone <mpilone@slac.com>                   
                                                                        
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or   
    (at your option) any later version.                                 
                                                                        
    This program is distributed in the hope that it will be useful,     
    but WITHOUT ANY WARRANTY; without even the implied warranty of      
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the        
    GNU General Public License for more details.                        
                                                                        
    You should have received a copy of the GNU General Public License   
    along with this program; if not, write to the Free Software         
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           
                                                                        
    As a special exception, permission is given to link this program    
    with any edition of Qt, and distribute the resulting executable,    
    without including the source code for Qt in the source distribution.
*/                                                                      

#include "kaddressbookiconview.h"

#include <qlayout.h>
#include <qiconview.h>
#include <qstringlist.h>
#include <kiconview.h>
#include <kiconloader.h>
#include <kglobal.h>
#include <kdebug.h>

#include "contactentrylist.h"
#include "contactentry.h"

////////////////////////////////
// AddresseeIconViewItem  (internal class)
class AddresseeIconViewItem : public KIconViewItem
{
  public:
    AddresseeIconViewItem(const KABC::Field::List &fields,
                          ContactEntryList *doc, const QString &uid, 
                          QIconView *parent)
      : KIconViewItem(parent), mFields( fields ), mDocument(doc), mUid(uid)
      {
          if ( mFields.isEmpty() ) {
            mFields = KABC::Field::defaultFields();
          }
          refresh();
      }
      
    const QString &uid() const { return mUid; }
    
    void refresh()
    {
        // Find the contact
        ContactEntry *ce = mDocument->find(mUid);
        
        if (ce)
        {
            setText( mFields.first()->value( ce->addressee() ) );
// Shouldn't this honor the user selections?
#if 0
            const QString *text = ce->find(mInternalAttributes[0]);
            if (text == 0)
                text = ce->find("File As");
            
            if (text == 0)
                text = ce->find("EMAIL");
                
            if (text == 0)
                setText("Uknown");
            else
                setText(*text);
#endif
        }
    }
    
  private:
    KABC::Field::List mFields;
    ContactEntryList *mDocument;
    QString mUid;
};

///////////////////////////////
// KAddressBookView

KAddressBookIconView::KAddressBookIconView(ContactEntryList *doc, QWidget *parent,
                                       const char *name)
    : KAddressBookView(doc, parent, name)
{
    mDocument = doc;
    
    // Init the GUI
    QVBoxLayout *layout = new QVBoxLayout(viewWidget());
    
    mIconView = new KIconView(viewWidget(), "mIconView");
    mIconView->setSelectionMode(QIconView::Extended);
    mIconView->setResizeMode(QIconView::Adjust);
    mIconView->setWordWrapIconText(true);
    mIconView->setMaxItemTextLength(8);
    mIconView->setItemsMovable(false);
    mIconView->setSorting(true, true);
    layout->addWidget(mIconView);
    
    // Connect up the signals
    connect(mIconView, SIGNAL(executed(QIconViewItem *)),
            this, SLOT(addresseeExecuted(QIconViewItem *)));
    connect(mIconView, SIGNAL(selectionChanged()),
            this, SLOT(addresseeSelected()));
}

KAddressBookIconView::~KAddressBookIconView()
{
}
    
QStringList KAddressBookIconView::selectedUids()
{
    QStringList uidList;
    QIconViewItem *item;
    AddresseeIconViewItem *aItem;
    
    for (item = mIconView->firstItem(); item; item = item->nextItem())
    {
        if (item->isSelected())
        {
            aItem = dynamic_cast<AddresseeIconViewItem*>(item);
            if (aItem)
                uidList << aItem->uid();
        }
    }
    
    return uidList;
}
    
void KAddressBookIconView::refresh(QString uid)
{
    QIconViewItem *item;
    AddresseeIconViewItem *aItem;
    
    if (uid == QString::null)
    {
        // Rebuild the view
        mIconView->clear();
        
        QStringList uidList = addresseeUids();
        QStringList::ConstIterator iter;
        for (iter = uidList.begin(); iter != uidList.end(); ++iter)
        {
            aItem = new AddresseeIconViewItem(fields(), mDocument, *iter,
                                              mIconView);
            aItem->setPixmap(KGlobal::iconLoader()->loadIcon("vcard",
                                                             KIcon::Desktop));
        }
        
        // by default nothing is selected
        emit selected(QString::null);
        
    }
    else
    {
        // Try to find the one to refresh
        bool found = false;
        for (item = mIconView->firstItem(); item && !found; 
             item = item->nextItem())
        {
            aItem = dynamic_cast<AddresseeIconViewItem*>(item);
            if ((aItem) && (aItem->uid() == uid))
            {
                aItem->refresh();
                found = true;
            }    
        }
    }
}

void KAddressBookIconView::setSelected(QString uid, bool selected)
{
    QIconViewItem *item;
    AddresseeIconViewItem *aItem;
    
    if (uid == QString::null)
    {
        mIconView->selectAll(selected);
    }
    else
    {
        bool found = false;
        for (item = mIconView->firstItem(); item && !found; 
             item = item->nextItem())
         {
             aItem = dynamic_cast<AddresseeIconViewItem*>(item);
             
             if ((aItem) && (aItem->uid() == uid))
             {
                 mIconView->setSelected(aItem, selected);
                 found = true;
             }
         }
    }
}
   
void KAddressBookIconView::addresseeExecuted(QIconViewItem *item)
{
    AddresseeIconViewItem *aItem = dynamic_cast<AddresseeIconViewItem*>(item);
    
    if (aItem)
        emit executed(aItem->uid());
}

void KAddressBookIconView::addresseeSelected()
{
    QIconViewItem *item;
    AddresseeIconViewItem *aItem;
    
    bool found = false;
    for (item = mIconView->firstItem(); item && !found; 
         item = item->nextItem())
    {
        if (item->isSelected())
        {
            aItem = dynamic_cast<AddresseeIconViewItem*>(item);
            emit selected(aItem->uid()); 
            found = true;
        }
    }
    
    if (!found)
        emit selected(QString::null);
}

void KAddressBookIconView::incrementalSearch(const QString &value, 
                                             KABC::Field *field)
{
  // Only supports searching on the first field, since that is all that
  // is visible
  if ( field->equals( fields().first() ) )
  {
    QIconViewItem *item = mIconView->findItem(value);
    
    if (item)
    {
      mIconView->setSelected(item, true, false);
      mIconView->ensureItemVisible(item);
    }
  }
}

#include "kaddressbookiconview.moc"
