/*
 * kaddressbookmain.cpp
 *
 * Copyright (C) 1999 Don Sanders <dsanders@kde.org>
 */
#include "browserentryeditor.h"

#include <qclipboard.h>

#include <kiconloader.h>
#include <klocale.h>
#include <kglobal.h>
#include <kmenubar.h>
#include <kconfig.h>
#include <kaccel.h>
#include <kdebug.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <kstandarddirs.h>
#include <kmessagebox.h>
#include <kprocess.h>
#include <kprotocolinfo.h>
#include <kedittoolbar.h>
#include <kkeydialog.h>

#include "kaddressbook.h"
#include "viewmanager.h"
#include "undo.h"
#include "kaddressbookmain.h"
#include "kaddressbookmain.moc"

KAddressBookMain::KAddressBookMain()
  : KMainWindow(0), DCOPObject("KAddressBookIface")
{
    setCaption( i18n( "Address Book Browser" ));

    mWidget = new KAddressBook( this, "KAddressBook" );
    connect( mWidget, SIGNAL( addresseeSelected( bool ) ),
             SLOT( addresseeSelected( bool ) ) );
    connect( mWidget, SIGNAL( modified( bool ) ),
             SLOT( modified( bool ) ) );
             
    mViewManager = mWidget->viewManager();
    connect( mViewManager, SIGNAL( viewConfigChanged(const QString &) ),
             SLOT( viewConfigChanged(const QString &) ) );
  
    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(mWidget);

    initActions();

    // we do want a status bar
    statusBar()->show();
  
    connect( QApplication::clipboard(), SIGNAL( dataChanged() ),
             SLOT( clipboardDataChanged() ) );
           
    // Tell the central widget to read its config
    mWidget->readConfig();
    
    // Read our own config
    KConfig *config = kapp->config();
    config->setGroup("Views");
    mActiveViewName = config->readEntry("Active");
    config->setGroup("MainWindow");
    mActionJumpBar->setChecked(config->readBoolEntry("JumpBar", false));
    mActionIncSearch->setChecked(config->readBoolEntry("IncSearch", true));
    
    // Set the defaults
    addresseeSelected(false);
    modified(false);
    quickToolsAction();

    // This needs to be done last, since it will try to tell the widget which
    // view to display.
    mActionViewList.setAutoDelete(true);
    initActionViewList();

    setAutoSaveSettings();
}

void KAddressBookMain::clipboardDataChanged()
{
    mActionPaste->setEnabled( !QApplication::clipboard()->text().isEmpty() );
}

void KAddressBookMain::initActions()
{
  // File menu
  mActionSave = new KAction(i18n("&Sync"), "filesave", CTRL+Key_S, mWidget, 
                             SLOT(save()), actionCollection(),"file_sync");
              
  new KAction(i18n("&New Contact"), "filenew", CTRL+Key_N, mWidget,
              SLOT(newAddressee()),actionCollection(),"file_new_contact");
              
  new KAction(i18n("&Distribution List..."), 0, mWidget,
              SLOT(slotDistributionList()), actionCollection(),
              "file_distribution_list");
              
  mActionMail = KStdAction::mail(mViewManager, SLOT(sendMail()), 
                                 actionCollection());
  mActionEditAddressee = new KAction(i18n("&Edit Contact..."), "edit", 0, 
                                     mWidget, SLOT(editAddressee()), 
                                     actionCollection(), "file_properties");

  new KAction(i18n("Import &KDE 2 Address Book"), 0, mWidget, 
              SLOT(importKDE2()), actionCollection(), "file_import_kde2");
  
  new KAction(i18n("Import vCard"), 0, mWidget, SLOT(importVCard()),
              actionCollection(), "file_import_vcard");
            
  new KAction(i18n("&Import List..."), 0, mWidget, SLOT(importCSV()),
              actionCollection(), "file_import_csv");
              
  new KAction(i18n("&Export List..."), 0, mWidget, SLOT(exportCSV()),
              actionCollection(), "file_export_csv");
              
  KStdAction::quit(this, SLOT(close()), actionCollection());

  // Edit menu
  mActionCut = KStdAction::cut(mViewManager, SLOT(cut()), actionCollection());

  mActionCopy = KStdAction::copy(mViewManager, SLOT(copy()), 
                                 actionCollection());

  mActionPaste = KStdAction::paste(mViewManager, SLOT(paste()), 
                                   actionCollection());
  clipboardDataChanged();

  KStdAction::selectAll(mViewManager, SLOT(setSelected()), actionCollection());

  mActionDelete = new KAction(i18n("&Delete Contact"), "editdelete", 0, 
                       mViewManager,
                       SLOT(deleteAddressee()), actionCollection(),
                       "edit_delete");

  mActionUndo = KStdAction::undo(mWidget, SLOT(undo()), actionCollection());
  mActionUndo->setEnabled(false);

  mActionRedo = KStdAction::redo(mWidget, SLOT(redo()), actionCollection());
  mActionRedo->setEnabled( false );

  // View menu
  new KAction(i18n("Modify View..."), "configure", 0, mViewManager,
              SLOT(modifyView()), actionCollection(),
              "view_modify");
  new KAction(i18n("Add View..."), "window_new", 0, mViewManager,
              SLOT(addView()), actionCollection(),
              "view_add");
  mActionDeleteView = new KAction(i18n("Delete View"), "delete", 0, 
              mViewManager,
              SLOT(deleteView()), actionCollection(),
              "view_delete");            
  new KAction(i18n("Refresh View"), "reload", 0, mViewManager,
              SLOT(refresh()), actionCollection(),
              "view_refresh");

  // Only enable LDAP lookup if we can handle the protocol
  if( KProtocolInfo::isKnownProtocol( KURL("ldap://localhost") )) {
    // LDAP button on toolbar
    new KAction(i18n("&Lookup addresses in directory"),
                "ldap_lookup", 0, mWidget, SLOT(slotOpenLDAPDialog()),
                actionCollection(),"ldap_lookup");
  }

  // Settings menu
  mActionJumpBar = new KToggleAction(i18n("Show Jump Bar"), "next", 0,
                                     this, SLOT(quickToolsAction()),
                                     actionCollection(), 
                                     "options_show_jump_bar");
  mActionIncSearch = new KToggleAction(i18n("Show Incremental Search"), "find",
                                     0, this, SLOT(quickToolsAction()),
                                     actionCollection(), 
                                     "options_show_inc_search");                                   
  KStdAction::preferences( mWidget, SLOT( configure() ), actionCollection());
  KStdAction::configureToolbars( this, SLOT( configureToolbars() ),
                                 actionCollection() );
  KStdAction::keyBindings(this, SLOT( configureKeys()), actionCollection() );
  
  createGUI( "kaddressbookui.rc", false );
  QObject::connect( guiFactory()->container("edit", this),
                   SIGNAL( aboutToShow() ), this, SLOT( updateEditMenu() ));
}

KAddressBookMain::~KAddressBookMain()
{
    mWidget->writeConfig();
    
    // Write our own config
    KConfig *config = kapp->config();
    
    config->setGroup("Views");
    config->writeEntry("Active", mActiveViewName);
    
    config->setGroup("MainWindow");
    config->writeEntry("JumpBar", mActionJumpBar->isChecked());
    config->writeEntry("IncSearch", mActionIncSearch->isChecked());
}

void KAddressBookMain::saveProperties(KConfig *)
{
  // the 'config' object points to the session managed
  // config file.  anything you write here will be available
  // later when this app is restored

  //what I want to save
  //windowsize
  //background image/underlining color/alternating color1,2
  //chosen fields
  //chosen fieldsWidths

  // e.g., config->writeEntry("key", var);
}

void KAddressBookMain::readProperties(KConfig *)
{
  // the 'config' object points to the session managed
  // config file.  this function is automatically called whenever
  // the app is being restored.  read in here whatever you wrote
  // in 'saveProperties'

  // e.g., var = config->readEntry("key");
}

void KAddressBookMain::updateEditMenu()
{
  UndoStack *undo = UndoStack::instance();
  RedoStack *redo = RedoStack::instance();

  if (undo->isEmpty())
    mActionUndo->setText( i18n( "Undo" ) );
  else
    mActionUndo->setText( i18n( "Undo" ) + " " + undo->top()->name() );
  mActionUndo->setEnabled( !undo->isEmpty() );

  if (!redo->top())
    mActionRedo->setText( i18n( "Redo" ) );
  else
    mActionRedo->setText( i18n( "Redo" ) + " " + redo->top()->name() );
  mActionRedo->setEnabled( !redo->isEmpty() );
}

void KAddressBookMain::addresseeSelected(bool selected)
{
    mActionCut->setEnabled(selected);
    mActionCopy->setEnabled(selected);
    mActionDelete->setEnabled(selected);
    mActionEditAddressee->setEnabled(selected);
    mActionMail->setEnabled(selected);
}

void KAddressBookMain::modified(bool mod)
{
    mActionSave->setEnabled(mod);
}

void KAddressBookMain::initActionViewList()
{
    // Create the view actions, and set the active view
    // Find the last active view
    QStringList viewNameList = mViewManager->viewNames();
    KToggleAction *viewAction = 0;

    // Just incast there is no active view!
    if (mActiveViewName.isEmpty() || 
       (viewNameList.contains(mActiveViewName) == 0))
        mActiveViewName = *(viewNameList).at(0);
    
    // unplug the current ones
    guiFactory()->unplugActionList(this, "view_loadedviews");

    // delete the current ones
    mActionViewList.clear();
    mActiveActionView = 0L;

    // Now find the active one, check the menu item, and raise it to the top
    QStringList::Iterator iter;
    QString viewName;
    for (iter = viewNameList.begin(); iter != viewNameList.end(); ++iter)
    {
        viewName = *iter;
        
        viewAction = new KToggleAction(viewName, QString::null, this,
                                 SLOT(selectViewAction()), actionCollection(),
                                 viewName.latin1());
        
        if (mActiveViewName == viewName)
        {
            mViewManager->setActiveView(viewName);
            
            viewAction->setChecked(true);
            mActiveActionView = viewAction;
        }
        
        mActionViewList.append(viewAction);
    }

    // Now append all the actions to the menu. 
    guiFactory()->plugActionList(this, "view_loadedviews", mActionViewList);
}

void KAddressBookMain::viewConfigChanged(const QString &newActive)
{
  if (!newActive.isEmpty())
  {
    mActiveViewName = newActive;
  }
  
  // we need to rebuild the actions
  initActionViewList();
  
  // Only enable delete if there is more than one view
  mActionDeleteView->setEnabled(mViewManager->viewNames().size() > 1);
}

void KAddressBookMain::selectViewAction()
{
    // See if we can find the selected action
  KToggleAction *action = 0;
  
  QString activatedName = sender()->name();
  QPtrListIterator<KAction> iter(mActionViewList);
  for (iter.toFirst(); iter.current(); ++iter)
  {
    action = dynamic_cast<KToggleAction*>(*iter);
        
    if (action->name() != activatedName)
      action->setChecked(false);
    else
    {
      mActiveActionView = action;
      mActiveActionView->setChecked(true);
      mActiveViewName = mActiveActionView->name();
     
      // Last, tell the main widget to set the view.
      mViewManager->setActiveView(mActiveViewName);
    }
  }
}

void KAddressBookMain::configureToolbars()
{
  saveMainWindowSettings( KGlobal::config(), "MainWindow" );

  KEditToolbar dlg(factory());
  connect(&dlg,SIGNAL(newToolbarConfig()),this,SLOT(slotNewToolbarConfig()));

  dlg.exec();
}

void KAddressBookMain::slotNewToolbarConfig()
{
  initActionViewList();

  applyMainWindowSettings( KGlobal::config(), "MainWindow" );
}

void KAddressBookMain::configureKeys()
{
  KKeyDialog::configureKeys(actionCollection(),xmlFile(),true,this);
}

void KAddressBookMain::quickToolsAction()
{
  mViewManager->setJumpButtonBarVisible(mActionJumpBar->isChecked());
  mViewManager->setIncrementalSearchVisible(mActionIncSearch->isChecked());
}
